/*********************************************************************
 *
 * Calculate and return no. of relation-preserving maps
 * use mindepth to maxdepth elements of domain,
 * terminate as soon as no. of trials exceeds cutoff
 *
 ********************************************************************/

#include "prog.h"

double
calculate (mindepth, maxdepth, cutoff)
     int mindepth, maxdepth;
     double cutoff;
{
  /* Calculate and return no. of relation-preserving maps:
   * use mindepth to maxdepth elements of domain,
   * terminate as soon as no. of trials exceeds cutoff
   */
  double count;
  int rho;
  register short rp, n, t, np;
  register int ii, numrels;

  trials = 0;
  count = 0;
  n = mindepth;
  t = 1;
  for (;;) {			/* loop calc_do1: */
    if (t <= SB) {
      trials = trials + 1;
      if (trials > cutoff) {
	goto calc_od1;
      }
      /* Start of relpres subroutine:
       * if relpres(psi,t,n) then rp=1
       * relpres(psi,t,n) is eqt to psi[n]=t; relpres(psi,n)
       * t is the test value for psi[n], relpres(psi,n-1) is true */

#include "relpres.c"
/* Uses np, rho, ii and numrels to calculate relpres(psi,t,n)
 * Returns result in rp
 */

      if (rp == 1) {
	if (n == maxdepth) {
	  count++;
	  if (DROP && (n == SA)) {
	    /* Test if current map is still relpres when the dropped
             * relation is re-inserted. Ie test if the current map
             * preserves the dropped relation (R).
             * Note that we can't use rel_to_A etc. since they are not
             * set up for relation R.
             */
	    psi[n] = t;
	    if (!relpres ()) {
	      (void) printf ("Map found which preserves all relations except %i\n", DROP);
	      print_map ();
	      exit (0);
	    }
	  }
	  t++;
	} else {
	  psi[n] = t;
	  n++;
	  t = 1;
	}
      } else {
	t++;
      }
    } else {			/* from (t <= SB) */
      n--;
      if (n < mindepth) {
	goto calc_od1;
      }
      t = psi[n] + 1;
    }				/* fi from (t <= SB) */
  }
calc_od1:
  return (count);
}				/* End of calculate(mindepth, maxdepth, cutoff) */

int
relpres ()
{
  /* return 1 if current map (psi) preserves relation R, 0 otherwise */
  int i, j, result;

  result = 1;
  for (i = 1; i <= SA; i++) {
    for (j = 1; j <= SA; j++) {
      /* If i and j are related by R in SA
       * then their images must be related by R in SB
       */
      if (RA[R][pi[i]][pi[j]] && !RB[R][psi[i]][psi[j]]) {
	(void) printf ("Elements %i and %i are related in A but %i and %i are not related in B\n", pi[i], pi[j], psi[i], psi[j]);
	result = 0;
      }
    }
  }
  return (result);
}				/* End of relpres () */
